<?php
namespace Datec\DatecAmino\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2013 
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use Datec\DatecAmino\Domain\Model\AminoCriteria;
use Datec\DatecAmino\Domain\Repository\AminoRepository;
use TYPO3\CMS\Core\Messaging\FlashMessage;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;
use TYPO3\CMS\Extbase\Mvc\Controller\ControllerContext;
use TYPO3\CMS\Extbase\Mvc\Exception\StopActionException;
use TYPO3\CMS\Extbase\Property\TypeConverter\PersistentObjectConverter;
use TYPO3\CMS\Extbase\Utility\LocalizationUtility;
use TYPO3\CMS\Fluid\View\StandaloneView;

/**
 * @package datec_amino
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 */
class AminoController extends ActionController {
	
	/**
	 * extKey
	 */
	protected $extKey;
	
	/**
	 * errors
	 */
	protected $errors;
	
	/**
	 * database
	 */
	protected $database;
	
	/**
	 * aminoRepository
	 * 
	 * @var AminoRepository
	 * */
	protected $aminoRepository;

    /**
     * @param AminoRepository $aminoRepository
     */
    public function injectAminoRepository(AminoRepository $aminoRepository)
    {
        $this->aminoRepository = $aminoRepository;
    }
	
	/**
	 * initialize current action
	 * @return void
	 */
	public function initializeAction() {
		$this->extKey = $this->request->getControllerExtensionKey();
		$this->checkConfiguration();
		
	}
	
	/**
	 * initialize searchForm action
	 *
	 * @return void
	 */
	protected function initializeSearchFormAction() {
		// allow all Properties to be mapped
		$mappingConfiguration = $this->arguments->getArgument('aminoCriteria')->getPropertyMappingConfiguration();
		$mappingConfiguration->allowAllProperties();
		$mappingConfiguration
            ->setTypeConverterOption(
                PersistentObjectConverter::class,
                PersistentObjectConverter::CONFIGURATION_CREATION_ALLOWED,
                TRUE
            );
	}
	
	/**
	 * initialize search action
	 *
	 * @return void
	 */
	protected function initializeSearchAction() {
		// allow all Properties to be mapped
		$mappingConfiguration = $this->arguments->getArgument('aminoCriteria')->getPropertyMappingConfiguration();
		$mappingConfiguration->allowAllProperties();
        $mappingConfiguration
            ->setTypeConverterOption(
                PersistentObjectConverter::class,
                PersistentObjectConverter::CONFIGURATION_CREATION_ALLOWED,
                TRUE
            );
	}
	
	/**
	 * initialize searchCompounded action
	 *
	 * @return void
	 */
	protected function initializeSearchCompoundedAction() {
		// allow all Properties to be mapped
		$mappingConfiguration = $this->arguments->getArgument('aminoCriteria')->getPropertyMappingConfiguration();
		$mappingConfiguration->allowAllProperties();
        $mappingConfiguration
            ->setTypeConverterOption(
                PersistentObjectConverter::class,
                PersistentObjectConverter::CONFIGURATION_CREATION_ALLOWED,
                TRUE
            );
	}

    /**
     * action searchForm
     *
     * @param AminoCriteria|null $aminoCriteria
     * @return void
     * @throws StopActionException
     */
	public function searchFormAction(AminoCriteria $aminoCriteria = NULL) {
		if (count($this->errors) > 0) {
			$this->forward('stop');
		}
		
		// supress technical error messages
		// will probably be handled differently with Typo3 6.3
		$msgs = $this->controllerContext->getFlashMessageQueue()->getAllMessagesAndFlush();
		foreach($msgs as $msg) {
			if ($msg->getMessage() != 'An error occurred while trying to call Datec\DatecAmino\Controller\AminoController->searchAction()') {
				$this->addFlashMessage($msg->getMessage(), $msg->getTitle(), $msg->getSeverity());
			}
		}
		
		if ($this->feUserAccess()) {
			$operantOptions = [
				'AND' => LocalizationUtility::translate('tx_datecamino_domain_model_aminoCriteria.operant.and',$this->extKey),
				'OR' => LocalizationUtility::translate('tx_datecamino_domain_model_aminoCriteria.operant.or',$this->extKey),
				'AND NOT' => LocalizationUtility::translate('tx_datecamino_domain_model_aminoCriteria.operant.not',$this->extKey)
			];
		} else {
            $operantOptions = null;
			$this->addFlashMessage(LocalizationUtility::translate('tx_datecamino.errors.noFeUser', $this->extKey), '', FlashMessage::ERROR);
		}
		$displayTables = GeneralUtility::trimExplode(',', $this->settings['displayTables']);
		$this->view->assign('settings',$this->settings);
		$this->view->assign('aminoCriteria', $aminoCriteria);
		$this->view->assign('operantOptions', $operantOptions);
		$this->view->assign('displayTables', $displayTables);
	}

    /**
     * action search
     *
     * splits to different searchable tables
     *
     * @param AminoCriteria $aminoCriteria
     * @return void
     * @throws StopActionException
     * @throws \TYPO3\CMS\Extbase\Mvc\Exception\UnsupportedRequestTypeException
     */
	public function searchAction(AminoCriteria $aminoCriteria) {
		if ($this->feUserAccess()) {
			foreach ($this->settings['tables'] as $tableModel => $targetTable) {
				if ($aminoCriteria->getTable() == $targetTable) {
					$tableModelUc = ucfirst($tableModel);
					$this->view->assign('tableModel', $tableModel);
					$this->view->assign('tableModelUc', $tableModelUc);
					
					$repositoryCallName = 'find'.$tableModelUc.'s';
					if (method_exists($this->aminoRepository, $repositoryCallName)) {
						$result = $this->aminoRepository->{$repositoryCallName}($aminoCriteria);
						if ($result[0] !== FALSE) {
							if(count($result[0])) {
								$searchWords = [$aminoCriteria->getSword1(), $aminoCriteria->getSword2()];
								$this->view->assign('results',$result[0]);
								$this->view->assign('resultsCount',$result[1]);
								$this->view->assign('searchWords', $searchWords);
								if(is_array($this->settings['ratings'])) {
									$this->view->assign('ratings',$this->settings['ratings']);
								}
							} else {
								$this->addFlashMessage(LocalizationUtility::translate('tx_datecamino.errors.no'.$tableModelUc,$this->extKey), '', FlashMessage::WARNING);
								$this->redirect('searchForm', 'Amino', 'DatecAmino', $this->request->getArguments());
							}
						} else {
							$this->addFlashMessage(LocalizationUtility::translate('tx_datecamino.errors.dbError',$this->extKey), '', FlashMessage::ERROR);
							$this->redirect('searchForm', 'Amino', 'DatecAmino', $this->request->getArguments());
						}
					} else {
						$this->addFlashMessage(LocalizationUtility::translate('tx_datecamino.errors.technicalError',$this->extKey). LocalizationUtility::translate('tx_datecamino.errors.repositoryMethodUnknown',$this->extKey, [$repositoryCallName]), '', FlashMessage::ERROR);
						$this->redirect('searchForm', 'Amino', 'DatecAmino', $this->request->getArguments());
					}
					
					$this->view->assign('settings',$this->settings);
					return;
				}
			}
			//did not forward and break before? table is not configurated
			$this->errors [] = LocalizationUtility::translate('tx_datecamino.errors.technicalError',$this->extKey). LocalizationUtility::translate('tx_datecamino.errors.configuration.tables_unknown',$this->extKey).$aminoCriteria->getTable();
		} else {
			$this->errors[] = LocalizationUtility::translate('tx_datecamino.errors.noFeUser', $this->extKey);
			$this->addFlashMessage(LocalizationUtility::translate('tx_datecamino.errors.noFeUser', $this->extKey), '', FlashMessage::ERROR);
		}
		var_dump($this->errors);
		if (count($this->errors)) {
			$this->forward('stop');
		}		
	}

    /**
     * action searchCompounded
     *
     * Compounded results for listing, outpus as JSON for usage in AJAX requests
     *
     * @param AminoCriteria $aminoCriteria
     * @return string JSON encoded result object
     */
	public function searchCompoundedAction(AminoCriteria $aminoCriteria) {
		$resultObj = new \stdClass();
        $this->checkConfiguration();
        //if ($this->feUserAccess()) {
        //$resultObj->aminoCriteria = $aminoCriteria->_getProperties();
        $resultObj->searchWords = $aminoCriteria->getSearchwords();
        $resultObj->sections = array();
        $resultObj->resultsView = '';

        // search all tables, compound result
        if (is_array($this->settings['tables'])) {
            $critTables = $aminoCriteria->getTables();
            if (count($critTables)) {
                $sectionIndex = 0;

                foreach ($this->settings['tables'] as $tableModel => $targetTable) {
                    if (in_array($tableModel, $critTables)) {
                        $tableModelUc = ucfirst($tableModel);
                        $repositoryCallName = 'find'.$tableModelUc.'s';
                        if (method_exists($this->aminoRepository, $repositoryCallName)) {
                            $result = $this->aminoRepository->{$repositoryCallName}($aminoCriteria);
                            if ($result[0] !== FALSE) {
                                if(count($result[0])) {
                                    header($_SERVER['SERVER_PROTOCOL']." 200 OK", true, 200);
                                    $resultObj->sections[$sectionIndex]->tableModel = $tableModel;
                                    $resultObj->sections[$sectionIndex]->tableModelUc = $tableModelUc;
                                    $resultObj->sections[$sectionIndex]->tableModelDisplayName = $this->settings['displayTableNames'][$tableModel];
                                    $resultObj->sections[$sectionIndex]->results = $this->feUserAccess() ? $result[0] : null; // only publish results with access, no access - only publish count
                                    $resultObj->sections[$sectionIndex]->resultsCount = $result[1];
                                    $resultObj->fullCount = isset($resultObj->fullCount) ? $resultObj->fullCount + $result[1] : $result[1];
                                } else {
                                    //header($_SERVER['SERVER_PROTOCOL']." 204 No Content", true, 204);
                                    //$this->errors[] = $tableModel;
                                    //$this->errors[] = $result[1];
                                    $this->errors[] = LocalizationUtility::translate('tx_datecamino.errors.no'.$tableModelUc,$this->extKey);
                                }
                            } else {
                                header($_SERVER['SERVER_PROTOCOL']." 500 Internal Server Error", true, 500);
                                //$this->errors[] = $tableModel;
                                //$this->errors[] = $result[1];
                                $this->errors[] = LocalizationUtility::translate('tx_datecamino.errors.dbError',$this->extKey);
                            }
                        } else {
                            header($_SERVER['SERVER_PROTOCOL']." 500 Internal Server Error", true, 500);
                            //$this->errors[] = $tableModel;
                            //$this->errors[] = $result[1];
                            $this->errors[] = LocalizationUtility::translate('tx_datecamino.errors.technicalError',$this->extKey). LocalizationUtility::translate('tx_datecamino.errors.repositoryMethodUnknown',$this->extKey,array($repositoryCallName));
                        }

                    }
                    $sectionIndex++;
                }
                // output settings
                $resultObj->settings = $this->settings;
                // for security, do not output
                unset($resultObj->settings['aminoServerHost']);
                unset($resultObj->settings['aminoServerUser']);
                unset($resultObj->settings['aminoServerPassword']);
                unset($resultObj->settings['allowedFeUserGroups']);
                // put ratings acessible on first level
                if(is_array($this->settings['ratings'])) {
                    $resultObj->ratings = $this->settings['ratings'];
                }
                // define number of sections
                $resultObj->sections['length'] = count($resultObj->sections);
                if (count($resultObj->sections)) {
                    // render all sections for view
                    $resultObj->resultsView = $this->renderFluidTemplate(
                        'Amino/SearchCompounded.html',
                        array('resultObj' => $resultObj),
                        null,
                        $this->getControllerContext()
                    );
                }
            } else {
                header($_SERVER['SERVER_PROTOCOL']." 204 No Content", true, 204);
                $this->errors[] = LocalizationUtility::translate('tx_datecamino.errors.noTables',$this->extKey);
            }
        } else {
            header($_SERVER['SERVER_PROTOCOL']." 500 Internal Server Error", true, 500);
            $this->errors[] = LocalizationUtility::translate('tx_datecamino.errors.technicalError',$this->extKey). LocalizationUtility::translate('tx_datecamino.errors.configuration.tables_unknown',$this->extKey);
        }
        /*} else {
            header($_SERVER['SERVER_PROTOCOL']." 403 Forbidden", true, 403);
            $this->errors[] = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.noFeUser', $this->extKey);
        }*/

        $resultObj->errors = $this->errors;

        return json_encode($resultObj);
	}
	
	/**
	 * action stop
	 *
	 * displays errors on page
	 *
	 * @return void
	 */
	public function stopAction() {
		if (isset($this->errors)) {
			foreach ($this->errors as $errorMsg) {
				$this->addFlashMessage($errorMsg, '', FlashMessage::ERROR);
			}
		}
	}

    /**
     * Returns the rendered fluid email template
     * @param string $template
     * @param array $assign
     * @param string $resourcePath
     * @param ControllerContext $controllerContext
     * @return string
     */
    public function renderFluidTemplate($template, Array $assign = array(), $resourcePath = NULL, $controllerContext = NULL) {
        $resourcePath = GeneralUtility::getFileAbsFileName($resourcePath === NULL ? 'EXT:datec_amino/Resources/Private/' : $resourcePath);

        /* @var $view StandaloneView */
        $view = GeneralUtility::makeInstance(StandaloneView::class);
        $view->setLayoutRootPaths([$resourcePath . 'Layouts/']);
        $view->setPartialRootPaths([$resourcePath . 'Partials/']);
        $view->setTemplatePathAndFilename($resourcePath . 'Templates/' . $template);
        $view->setControllerContext($controllerContext);
        $view->assignMultiple($assign);

        return $view->render();
    }
	
	/**
	 * if configured, the access will be granted to frontent users logged in with the allowed groups
	 * note: with this, only logged in users can use this plugin (as required by the AMINO projekt)
	 * make sure to configure 'allowedFeUserGroups' with a comma-seperated list of your pharmaceutical personel's FE-groups
	 * 
	 * @return boolean
	 */
	private function feUserAccess() {
		$access = FALSE;
		if (!isset($this->settings['allowedFeUserGroups']) || empty($this->settings['allowedFeUserGroups'])) {
			$this->errors[] = LocalizationUtility::translate('tx_datecamino.errors.technicalError',$this->extKey). LocalizationUtility::translate('tx_datecamino.errors.configuration.allowedFeUserGroups',$this->extKey);
			return $access;
		}
		foreach (explode(',', $this->settings['allowedFeUserGroups']) as $allowedUsergroup) {
			$access = in_array($allowedUsergroup, explode(',', $GLOBALS['TSFE']->gr_list)); // check with allowed groups list against users access list
			if ($access) {
				break;
			}
		}
		return $access;
	}
	
	/**
	 * check plugin configuration
	 *
	 * @return void
	 */
	private function checkConfiguration() {
		if (!isset($this->settings['aminoServerHost']) || empty($this->settings['aminoServerHost'])) {
			$this->errors[] = LocalizationUtility::translate('tx_datecamino.errors.technicalError',$this->extKey). LocalizationUtility::translate('tx_datecamino.errors.configuration.aminoServerHost',$this->extKey);
		}
		if (!isset($this->settings['aminoServerUser']) || empty($this->settings['aminoServerUser'])) {
			$this->errors[] = LocalizationUtility::translate('tx_datecamino.errors.technicalError',$this->extKey). LocalizationUtility::translate('tx_datecamino.errors.configuration.aminoServerUser',$this->extKey);
		}
		if (!isset($this->settings['aminoServerPassword']) || empty($this->settings['aminoServerPassword'])) {
			$this->errors[] = LocalizationUtility::translate('tx_datecamino.errors.technicalError',$this->extKey). LocalizationUtility::translate('tx_datecamino.errors.configuration.aminoServerPassword',$this->extKey);
		}
		if (!isset($this->settings['tables']) || empty($this->settings['tables'])) {
			$this->errors[] = LocalizationUtility::translate('tx_datecamino.errors.technicalError',$this->extKey). LocalizationUtility::translate('tx_datecamino.errors.configuration.tables',$this->extKey);
		}
		if (!isset($this->settings['allowedFeUserGroups']) || empty($this->settings['allowedFeUserGroups'])) {
			$this->errors[] = LocalizationUtility::translate('tx_datecamino.errors.technicalError',$this->extKey). LocalizationUtility::translate('tx_datecamino.errors.configuration.allowedFeUserGroups',$this->extKey);
		}
	}

}